=== modified file 'unity-shared/BGHash.cpp'
--- unity-shared/BGHash.cpp	2012-11-06 18:19:09 +0000
+++ unity-shared/BGHash.cpp	2013-09-24 20:14:21 +0000
@@ -20,7 +20,8 @@
 #include "BGHash.h"
 #include <gdk/gdkx.h>
 #include <NuxCore/Logger.h>
-#include "unity-shared/UBusMessages.h"
+#include "unity-shared/AnimationUtils.h"
+#include "unity-shared/WindowManager.h"
 
 #ifndef XA_STRING
 #define XA_STRING ((Atom) 31)
@@ -31,22 +32,30 @@
 namespace unity
 {
 
+namespace
+{
+  Atom COLORS_ATOM = 0;
+  const unsigned TRANSITION_DURATION = 500;
+}
+
 BGHash::BGHash()
-  : current_color_(unity::colors::Aubergine)
+  : transition_animator_(TRANSITION_DURATION)
+  , override_color_(nux::color::Transparent)
 {
-  transition_animator_.SetDuration(500);
-  override_color_.alpha = 0.0f;
-
+  COLORS_ATOM = gdk_x11_get_xatom_by_name("_GNOME_BACKGROUND_REPRESENTATIVE_COLORS");
   transition_animator_.updated.connect(sigc::mem_fun(this, &BGHash::OnTransitionUpdated));
-  ubus_manager_.RegisterInterest(UBUS_BACKGROUND_REQUEST_COLOUR_EMIT, [&](GVariant *) { DoUbusColorEmit(); } );
-
+  WindowManager::Default().average_color = unity::colors::Aubergine;
   RefreshColor();
 }
 
+uint64_t BGHash::ColorAtomId() const
+{
+  return COLORS_ATOM;
+}
+
 void BGHash::OverrideColor(nux::Color const& color)
 {
   override_color_ = color;
-
   RefreshColor();
 }
 
@@ -58,23 +67,23 @@
     return;
   }
 
+  // XXX: move this part to PluginAdapter
   Atom         real_type;
   gint         result;
   gint         real_format;
   gulong       items_read;
   gulong       items_left;
   gchar*       colors;
-  Atom         representative_colors_atom;
   Display*     display;
   GdkRGBA      color_gdk;
 
-  representative_colors_atom = gdk_x11_get_xatom_by_name("_GNOME_BACKGROUND_REPRESENTATIVE_COLORS");
-  display = gdk_x11_display_get_xdisplay(gdk_display_get_default ());
+  colors = nullptr;
+  display = gdk_x11_display_get_xdisplay(gdk_display_get_default());
 
   gdk_error_trap_push();
   result = XGetWindowProperty (display,
              GDK_ROOT_WINDOW(),
-             representative_colors_atom,
+             COLORS_ATOM,
              0L,
              G_MAXLONG,
              False,
@@ -90,63 +99,30 @@
   if (result == Success && items_read)
   {
     gdk_rgba_parse(&color_gdk, colors);
-    nux::Color new_color(color_gdk.red,
-                         color_gdk.green,
-                         color_gdk.blue,
-                         1.0f);
+    nux::Color new_color(color_gdk.red, color_gdk.green, color_gdk.blue, 1.0f);
     TransitionToNewColor(MatchColor(new_color));
-    XFree (colors);
   }
+
+  XFree(colors);
 }
 
 void BGHash::TransitionToNewColor(nux::color::Color const& new_color)
 {
-  if (new_color == current_color_)
-    return;
-
-  LOG_DEBUG(logger) << "transitioning from: " << current_color_.red << " to " << new_color.red;
-
-  transition_animator_.SetStartValue(current_color_);
-  transition_animator_.SetFinishValue(new_color);
-  transition_animator_.Stop();
-  transition_animator_.Start();
+  auto const& current_color = WindowManager::Default().average_color();
+  LOG_DEBUG(logger) << "transitioning from: " << current_color.red << " to " << new_color.red;
+
+  animation::StartOrReverse(transition_animator_, current_color, new_color);
 }
 
 void BGHash::OnTransitionUpdated(nux::Color const& new_color)
 {
-  current_color_ = new_color;
-  DoUbusColorEmit();
-}
-
-void BGHash::DoUbusColorEmit()
-{
-  ubus_manager_.SendMessage(UBUS_BACKGROUND_COLOR_CHANGED,
-                            g_variant_new ("(dddd)",
-                                           current_color_.red,
-                                           current_color_.green,
-                                           current_color_.blue,
-                                           current_color_.alpha));
+  WindowManager::Default().average_color = new_color;
 }
 
 nux::Color BGHash::MatchColor(nux::Color const& base_color) const
 {
-  nux::Color colors[12];
-
-  colors[ 0] = nux::Color (0x540e44);
-  colors[ 1] = nux::Color (0x6e0b2a);
-  colors[ 2] = nux::Color (0x841617);
-  colors[ 3] = nux::Color (0x84371b);
-  colors[ 4] = nux::Color (0x864d20);
-  colors[ 5] = nux::Color (0x857f31);
-  colors[ 6] = nux::Color (0x1d6331);
-  colors[ 7] = nux::Color (0x11582e);
-  colors[ 8] = nux::Color (0x0e5955);
-  colors[ 9] = nux::Color (0x192b59);
-  colors[10] = nux::Color (0x1b134c);
-  colors[11] = nux::Color (0x2c0d46);
-
   nux::Color chosen_color;
-  nux::color::HueSaturationValue base_hsv (base_color);
+  nux::color::HueSaturationValue base_hsv(base_color);
 
   if (base_hsv.saturation < 0.08)
   {
@@ -157,27 +133,34 @@
   }
   else
   {
+    std::vector<nux::Color> colors = {
+      nux::Color(0x540e44), nux::Color(0x6e0b2a), nux::Color(0x841617),
+      nux::Color(0x84371b), nux::Color(0x864d20), nux::Color(0x857f31),
+      nux::Color(0x1d6331), nux::Color(0x11582e), nux::Color(0x0e5955),
+      nux::Color(0x192b59), nux::Color(0x1b134c), nux::Color(0x2c0d46),
+    };
+
     float closest_diff = 200.0f;
     LOG_DEBUG (logger) << "got a colour image";
     // full colour image
-    for (int i = 0; i < 11; i++)
+    for (auto const& color : colors)
     {
-      nux::color::HueSaturationValue comparison_hsv (colors[i]);
+      nux::color::HueSaturationValue comparison_hsv(color);
       float color_diff = fabs(base_hsv.hue - comparison_hsv.hue);
 
       if (color_diff < closest_diff)
       {
-        chosen_color = colors[i];
+        chosen_color = color;
         closest_diff = color_diff;
       }
     }
 
-    nux::color::HueSaturationValue hsv_color (chosen_color);
+    nux::color::HueSaturationValue hsv_color(chosen_color);
     hsv_color.saturation = std::min(base_hsv.saturation, hsv_color.saturation);
     hsv_color.saturation *= (2.0f - hsv_color.saturation);
-    hsv_color.value = std::min(std::min(base_hsv.value, hsv_color.value), 0.26f);
+    hsv_color.value = std::min({base_hsv.value, hsv_color.value, 0.26f});
 
-    chosen_color = nux::Color (nux::color::RedGreenBlue(hsv_color));
+    chosen_color = nux::Color(nux::color::RedGreenBlue(hsv_color));
     chosen_color.alpha = 0.72f;
   }
 
@@ -188,9 +171,9 @@
   return chosen_color;
 }
 
-nux::Color const& BGHash::CurrentColor() const
+nux::Color BGHash::CurrentColor() const
 {
-  return current_color_;
+  return WindowManager::Default().average_color();
 }
 
 }

=== modified file 'unity-shared/BGHash.h'
--- unity-shared/BGHash.h	2012-08-15 04:58:17 +0000
+++ unity-shared/BGHash.h	2013-09-24 20:14:21 +0000
@@ -21,7 +21,6 @@
 
 #include <NuxCore/Animation.h>
 #include <Nux/Nux.h>
-#include "UBusWrapper.h"
 
 namespace unity {
 namespace colors {
@@ -36,23 +35,19 @@
   public:
     BGHash();
 
-    nux::Color const& CurrentColor() const;
+    nux::Color CurrentColor() const;
+    uint64_t ColorAtomId() const;
     void RefreshColor();
     void OverrideColor(nux::Color const& color);
 
   private:
     void OnTransitionUpdated(nux::Color const& new_color);
-    void DoUbusColorEmit();
     void TransitionToNewColor(nux::Color const& new_color);
     nux::Color MatchColor(nux::Color const& base_color) const;
 
   private:
     nux::animation::AnimateValue<nux::Color> transition_animator_;
-
-    nux::Color current_color_; // the current colour, including steps in transitions
     nux::Color override_color_;
-
-    UBusManager ubus_manager_;
   };
 };
 

